/**
 * Mapasuta
 *
 * Copyright (C) 2006 Dominik Winter
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 **/
package mapasuta.build.maven.plugin.oaw;

import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.maven.artifact.Artifact;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.project.MavenProject;
import org.codehaus.plexus.util.StringUtils;
import org.openarchitectureware.workflow.WorkflowRunner;
import org.openarchitectureware.workflow.monitor.NullProgressMonitor;

/**
 * @author Dominik Winter
 */
abstract class AbstractOawMojo extends AbstractMojo
{

   /**
    * Workflow file to run.
    * 
    * @parameter
    * @required
    */
   private String workflowFile = null;
   /**
    * Workflow parameters.
    * 
    * @parameter
    */
   private Map parameters = new HashMap();
   /**
    * Resource folders.
    * 
    * @parameter
    */
   private File resources[] = null;
   /**
    * @parameter expression="${project}"
    * @required
    * @readonly
    */
   private MavenProject project = null;

   /**
    * @parameter default-value="${project.artifacts}"
    * @required
    * @readonly
    */
   private Set dependencies;

   public void execute() throws MojoExecutionException, MojoFailureException
   {
      LogAdapter.setMavenLog(getLog());
      System.setProperty("org.apache.commons.logging.Log", LogAdapter.class.getName());
      Log log = LogFactory.getLog(getClass());

      ClassLoader cl =
            URLClassLoader.newInstance(prepareClasspath(), this.getClass()
               .getClassLoader());
      Thread.currentThread().setContextClassLoader(cl);

      WorkflowRunner runner = new WorkflowRunner();
      Map parms = getParameters();
      parms.put("outputDir", getOutputDir());
      if (runner.run(workflowFile,
                     new NullProgressMonitor(),
                     parms,
                     new HashMap<String, String>()))
      {
         setSourceRoot(getOutputDir());
      }
      else
      {
         String errors = ((LogAdapter)log).getErrors();
         List exceptions = ((LogAdapter)log).getExceptions();
         if (exceptions.size() > 0) throw new MojoExecutionException(errors,
                                                                     (Exception)exceptions.get(exceptions.size() - 1));
         else throw new MojoExecutionException(errors);
      }

   }

   private URL[] prepareClasspath() throws MojoExecutionException
   {
      List<URL> cpList = new ArrayList<URL>();

      try
      {
         if (getDependencies() != null) for (Object object: getDependencies())
         {
            Artifact atf = (Artifact)object;
            URL url = atf.getFile().toURL();
            getLog().debug("Adding dependency " + url + " to classpath.");
            cpList.add(url);
         }

         if (getResources() != null)
            for (File resobj : getResources()) {
               URL resurl = resobj.toURL();
               getLog().debug("Adding resource path " + resurl + " to classpath.");
               cpList.add(resurl);
            }
         if (getProject() != null) {
            String buildout = getProject().getBuild().getOutputDirectory();
            String testout = getProject().getBuild().getTestOutputDirectory();
            if (StringUtils.isNotEmpty(buildout)) {
               getLog().debug("Adding compile output path " + buildout + " to classpath.");
               cpList.add(new URL("file:"+buildout+"/"));
            }
            if (StringUtils.isNotEmpty(testout)) {
               getLog().debug("Adding test output path " + testout + " to classpath.");
               cpList.add(new URL("file:"+testout+"/"));
            }
         }
      }
      catch (Exception e)
      {
         throw new MojoExecutionException(e.getMessage(), e);
      }
      return cpList.toArray(new URL[cpList.size()]);
   }

   /**
    * @return Returns the parameters.
    */
   protected final Map getParameters()
   {
      return parameters;
   }

   /**
    * @param parameters
    *           The parameters to set.
    */
   protected final void setParameters(Map parameters)
   {
      this.parameters = parameters;
   }

   /**
    * @return Returns the workflowFile.
    */
   protected final String getWorkflowFile()
   {
      return workflowFile;
   }

   /**
    * @param workflowFile
    *           The workflowFile to set.
    */
   protected final void setWorkflowFile(String workflowFile)
   {
      this.workflowFile = workflowFile;
   }

   /**
    * @return the project
    */
   protected final MavenProject getProject()
   {
      return project;
   }

   /**
    * @return the outputDir
    */
   protected abstract String getOutputDir();

   abstract protected void setSourceRoot(String sourceRoot);

   /**
    * @return the dependencies
    */
   protected final Set getDependencies()
   {
      return dependencies;
   }

   /**
    * @param dependencies
    *           the dependencies to set
    */
   protected final void setDependencies(Set dependencies)
   {
      this.dependencies = dependencies;
   }

   /**
    * @return the resources
    */
   protected File[] getResources()
   {
      return resources;
   }

   /**
    * @param resources the resources to set
    */
   protected void setResources(File[] resources)
   {
      this.resources = resources;
   }

}
